@@ -42,6 +42,35 @@ class TestTranslationString(unittest.TestCase):
         self.assertEqual(inst.mapping, 'mapping')
         self.assertEqual(inst.domain, 'domain')
 
+    def test_format_no_exising_mapping(self):
+        inst = self._makeOne('msgid', domain='domain', default='default')
+        mapping = {'foo': 'bar'}
+        new_inst = inst % mapping
+        self.assertTrue(new_inst is not inst)
+        self.assertEqual(new_inst, 'msgid')
+        self.assertEqual(new_inst.default, 'default')
+        self.assertEqual(new_inst.domain, 'domain')
+        self.assertEqual(new_inst.mapping, mapping)
+        # Make sure a copy is made to prevent changes in the original
+        # dictionary from modifying our mapping.
+        self.assertTrue(new_inst.mapping is not mapping)
+
+    def test_format_extend_existing_mapping(self):
+        inst = self._makeOne('msgid', domain='domain', default='default',
+                mapping={'one': '1', 'two': '3'})
+        new_inst = inst % {'two': '2', 'three': '3'}
+        self.assertEqual(
+                new_inst.mapping,
+                {'one': '1', 'two': '2', 'three': '3'})
+        # Make sure original is not changed
+        self.assertEqual(inst.mapping, {'one': '1', 'two': '3'})
+
+    def test_format_required_dict_argument(self):
+        import operator
+        inst = self._makeOne('msgid', domain='domain', default='default')
+        self.assertRaises(ValueError, operator.mod, inst, ('foo',))
+
+
     def test_interpolate_substitution(self):
         mapping = {"name": "Zope", "version": 3}
         inst = self._makeOne('This is $name version ${version}.',
@@ -122,6 +151,11 @@ class TestChameleonTranslate(unittest.TestCase):
         result = translate(None)
         self.assertEqual(result, None)
 
+    def test_msgid_not_none_not_string(self):
+        translate = self._makeOne(None)
+        result = translate(True)
+        self.assertEqual(result, 'True')
+
     def test_chameleon_default_marker_returned(self):
         # Chameleon uses a special StringMarker class as default value so it
         # can detect missing translations.
